"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatAgentService = void 0;
const tslib_1 = require("tslib");
const flatMap_1 = tslib_1.__importDefault(require("lodash/flatMap"));
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const chat_api_service_1 = require("./chat.api.service");
let ChatAgentService = class ChatAgentService extends ide_core_common_1.Disposable {
    constructor() {
        super();
        this.agents = new Map();
        this._onDidChangeAgents = new ide_core_common_1.Emitter();
        this.onDidChangeAgents = this._onDidChangeAgents.event;
        this._onDidSendMessage = new ide_core_common_1.Emitter();
        this.onDidSendMessage = this._onDidSendMessage.event;
        this.addDispose(this._onDidChangeAgents);
    }
    registerAgent(agent) {
        if (this.agents.has(agent.id)) {
            throw new Error(`Already registered an agent with id ${agent.id}`);
        }
        this.agents.set(agent.id, { agent, commands: [] });
        this._onDidChangeAgents.fire();
        return (0, ide_core_common_1.toDisposable)(() => {
            if (this.agents.delete(agent.id)) {
                this._onDidChangeAgents.fire();
            }
        });
    }
    async updateAgent(id, updateMetadata) {
        const data = this.agents.get(id);
        if (!data) {
            throw new Error(`No agent with id ${id} registered`);
        }
        data.agent.metadata = Object.assign(Object.assign({}, data.agent.metadata), updateMetadata);
        data.agent.provideSlashCommands(ide_core_common_1.CancellationToken.None).then((commands) => {
            data.commands = commands;
        });
        if (updateMetadata.isDefault) {
            this.defaultAgentId = id;
        }
        const chatWelcomeMessage = await data.agent.provideChatWelcomeMessage(ide_core_common_1.CancellationToken.None);
        if (chatWelcomeMessage) {
            const { content, sampleQuestions } = chatWelcomeMessage;
            this.chatFeatureRegistry.registerWelcome(content, sampleQuestions);
        }
        this._onDidChangeAgents.fire();
    }
    getAgents() {
        return Array.from(this.agents.values(), (v) => v.agent);
    }
    hasAgent(id) {
        return this.agents.has(id);
    }
    getDefaultAgentId() {
        return this.defaultAgentId;
    }
    getAgent(id) {
        const data = this.agents.get(id);
        return data === null || data === void 0 ? void 0 : data.agent;
    }
    async invokeAgent(id, request, progress, history, token) {
        const data = this.agents.get(id);
        if (!data) {
            throw new Error(`No agent with id ${id}`);
        }
        const result = await data.agent.invoke(request, progress, history, token);
        return result;
    }
    populateChatInput(id, message) {
        this.aiChatService.sendMessage(Object.assign(Object.assign({}, message), { agentId: id, immediate: false }));
    }
    getCommands() {
        return (0, flatMap_1.default)(Array.from(this.agents.values(), ({ agent, commands }) => commands.map((c) => (Object.assign(Object.assign({}, c), { agentId: agent.id })))));
    }
    async getFollowups(id, sessionId, token) {
        const data = this.agents.get(id);
        if (!data) {
            throw new Error(`No agent with id ${id}`);
        }
        if (!data.agent.provideFollowups) {
            return [];
        }
        return data.agent.provideFollowups(sessionId, token);
    }
    async getSampleQuestions(id, token) {
        const data = this.agents.get(id);
        if (!data) {
            throw new Error(`No agent with id ${id}`);
        }
        if (!data.agent.provideSampleQuestions) {
            return [];
        }
        return data.agent.provideSampleQuestions(token);
    }
    async getAllSampleQuestions() {
        const result = await Promise.all(Array.from(this.agents.values()).map(async ({ agent }) => {
            try {
                return await this.getSampleQuestions(agent.id, ide_core_common_1.CancellationToken.None);
            }
            catch (err) {
                this.logger.error(err);
                return [];
            }
        }));
        return (0, flatMap_1.default)(result);
    }
    sendMessage(chunk) {
        this._onDidSendMessage.fire(chunk);
    }
    parseMessage(value, currentAgentId) {
        var _a;
        const parsedInfo = {
            agentId: '',
            command: '',
            message: value,
        };
        let useAgentId = currentAgentId;
        const agents = this.getAgents();
        const agentIdReg = new RegExp(`^@(${agents.map((a) => a.id).join('|')})(?:\\s+|$)`, 'i');
        const agentIdMatch = parsedInfo.message.match(agentIdReg);
        if (agentIdMatch) {
            const matchedAgent = agents.find((a) => a.id.toLowerCase() === agentIdMatch[1].toLowerCase());
            if (!matchedAgent) {
                return parsedInfo;
            }
            useAgentId = matchedAgent.id;
            parsedInfo.agentId = useAgentId;
            parsedInfo.message = parsedInfo.message.replace(agentIdMatch[0], '');
        }
        if (useAgentId) {
            const commands = (_a = this.agents.get(useAgentId)) === null || _a === void 0 ? void 0 : _a.commands;
            if (commands === null || commands === void 0 ? void 0 : commands.length) {
                const commandReg = new RegExp(`^/\\s?(${commands.map((c) => c.name).join('|')})(?:\\s+|$)`, 'i');
                const commandMatch = parsedInfo.message.match(commandReg);
                if (commandMatch) {
                    const matchedCommand = commands.find((c) => c.name.toLowerCase() === commandMatch[1].toLowerCase());
                    if (!matchedCommand) {
                        return parsedInfo;
                    }
                    parsedInfo.command = matchedCommand.name;
                    parsedInfo.message = parsedInfo.message.replace(commandMatch[0], '');
                }
            }
        }
        return parsedInfo;
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ILogger),
    tslib_1.__metadata("design:type", Object)
], ChatAgentService.prototype, "logger", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ChatServiceToken),
    tslib_1.__metadata("design:type", chat_api_service_1.ChatService)
], ChatAgentService.prototype, "aiChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", Object)
], ChatAgentService.prototype, "chatFeatureRegistry", void 0);
ChatAgentService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], ChatAgentService);
exports.ChatAgentService = ChatAgentService;
//# sourceMappingURL=chat-agent.service.js.map